% ================================================================================= % 
% plottogether.m 
%
% Loads impulse response functions from the baseline the signal extraction
% and the interest rate smoothing model and plots them together
%
% Chris Boehm 
% February 2014
% =========================================================================

clear all

load base_irfs
load SW_irfs
load IS_irfs

set(0,'defaulttextInterpreter','latex')
% set(0,'defaulttextInterpreter','none')
% set(gca, 'FontName', 'Calibri')

fig3 = figure(3);
  
 subplot(2,3,1)
 plot(dat,base_irfs_re(:,1),'b-', 'LineWidth', 1.5); 
 hold on
 plot(dat,SW_irfs_re(:,1),'r-.', 'LineWidth', 1.5); 
 plot(dat,IS_irfs_re(:,1),'k--', 'LineWidth', 1.5); 
 hold off
 line([0 20],[0 0],'Color','black', 'LineWidth', 0.5)
 legend({'TR1','TR2','TR3'},'Location','northeast','Interpreter','latex','FontSize',12)
 title('Output Gap','FontSize',12)
 ylabel('Efficient Rate Shock','FontSize',12)
 
 subplot(2,3,2)
 plot(dat,4*base_irfs_re(:,2),'b-', 'LineWidth', 1.5); 
 hold on
 plot(dat,4*SW_irfs_re(:,2),'r-.', 'LineWidth', 1.5); 
 plot(dat,4*IS_irfs_re(:,2),'k--', 'LineWidth', 1.5); 
 hold off
 line([0 20],[0 0],'Color','black', 'LineWidth', 0.5) 
 title('Inflation','FontSize',12)

 
 subplot(2,3,3)
 plot(dat,4*base_irfs_re(:,3),'b-', 'LineWidth', 1.5); 
 hold on
 plot(dat,4*SW_irfs_re(:,3),'r-.', 'LineWidth', 1.5); 
 plot(dat,4*IS_irfs_re(:,3),'k--', 'LineWidth', 1.5); 
 hold off
 line([0 20],[0 0],'Color','black', 'LineWidth', 0.5) 
 title('Nominal Interest Rate','FontSize',12)
 
 % Cost push shock
 
 subplot(2,3,4)
 plot(dat,base_irfs_u(:,1),'b-', 'LineWidth', 1.5); 
 hold on
 plot(dat,SW_irfs_u(:,1),'r-.', 'LineWidth', 1.5); 
 plot(dat,IS_irfs_u(:,1),'k--', 'LineWidth', 1.5); 
 hold off
 ylabel('Cost Push Shock','FontSize',12)
 axis([0 20 -0.8 0.1])
 line([0 20],[0 0],'Color','black', 'LineWidth', 0.5)
 title('Output Gap','FontSize',12)
 xlabel('Quarter','FontSize',12)
 
 subplot(2,3,5)
 plot(dat,4*base_irfs_u(:,2),'b-', 'LineWidth', 1.5); 
 hold on
 plot(dat,4*SW_irfs_u(:,2),'r-.', 'LineWidth', 1.5); 
 plot(dat,4*IS_irfs_u(:,2),'k--', 'LineWidth', 1.5); 
 hold off
 line([0 20],[0 0],'Color','black', 'LineWidth', 0.5)
 axis([0 20 -0.3 1.5])
 title('Inflation','FontSize',12)
 xlabel('Quarter','FontSize',12)
 
 subplot(2,3,6)
 plot(dat,4*base_irfs_u(:,3),'b-', 'LineWidth', 1.5); 
 hold on
 plot(dat,4*SW_irfs_u(:,3),'r-.', 'LineWidth', 1.5); 
 plot(dat,4*IS_irfs_u(:,3),'k--', 'LineWidth', 1.5); 
 hold off
 line([0 20],[0 0],'Color','black', 'LineWidth', 0.5)
 title('Nominal Interest Rate','FontSize',12)
 xlabel('Quarter','FontSize',12)
 
 set(fig3, 'PaperUnits', 'inches');
 scale = 2.5;
 x_width=scale*4.5; y_width=scale*3;
 set(fig3, 'PaperPosition', [0 0 x_width y_width]); 
 print('output\figure_3','-dpng','-r600') 
 
 fig4 = figure(4);
 % Measurement error in output shock
 
 subplot(2,3,1)
 plot(dat,base_irfs_my(:,1),'b-', 'LineWidth', 1.5); 
 hold on
 plot(dat,SW_irfs_my(:,1),'r-.', 'LineWidth', 1.5); 
 plot(dat,IS_irfs_my(:,1),'k--', 'LineWidth', 1.5); 
 hold off
 line([0 20],[0 0],'Color','black', 'LineWidth', 0.5)
 axis([0 20 -0.25 0.05])
 ylabel('Measurment Error Shock (Output)','FontSize',12)
 legend({'TR1','TR2','TR3'},'Location','southeast','Interpreter','latex','FontSize',12)
title('Output Gap','FontSize',12)
 
 subplot(2,3,2)
 plot(dat,4*base_irfs_my(:,2),'b-', 'LineWidth', 1.5); 
 hold on
 plot(dat,4*SW_irfs_my(:,2),'r-.', 'LineWidth', 1.5); 
 plot(dat,4*IS_irfs_my(:,2),'k--', 'LineWidth', 1.5); 
 hold off
 line([0 20],[0 0],'Color','black', 'LineWidth', 0.5) 
 axis([0 20 -1.0 0.1])
 title('Inflation','FontSize',12)
 
 subplot(2,3,3)
 plot(dat,4*base_irfs_my(:,3),'b-', 'LineWidth', 1.5); 
 hold on
 plot(dat,4*SW_irfs_my(:,3),'r-.', 'LineWidth', 1.5); 
 plot(dat,4*IS_irfs_my(:,3),'k--', 'LineWidth', 1.5); 
 hold off
 line([0 20],[0 0],'Color','black', 'LineWidth', 0.5) 
 axis([0 20 -0.4 0.1])
 title('Nominal Interest Rate','FontSize',12)
 
 % Measurement error in inflation shock
 
 subplot(2,3,4)
 plot(dat,base_irfs_mpi(:,1),'b-', 'LineWidth', 1.5); 
 hold on
 plot(dat,SW_irfs_mpi(:,1),'r-.', 'LineWidth', 1.5); 
 plot(dat,IS_irfs_mpi(:,1),'k--', 'LineWidth', 1.5); 
 hold off
 line([0 20],[0 0],'Color','black', 'LineWidth', 0.5) 
 axis([0 20 -0.6 0.1])
 ylabel('Measurment Error Shock (Inflation)','FontSize',12)
 xlabel('Quarter','FontSize',12)
 title('Output Gap','FontSize',12)
 
 subplot(2,3,5)
 plot(dat,4*base_irfs_mpi(:,2),'b-', 'LineWidth', 1.5); 
 hold on
 plot(dat,4*SW_irfs_mpi(:,2),'r-.', 'LineWidth', 1.5); 
 plot(dat,4*IS_irfs_mpi(:,2),'k--', 'LineWidth', 1.5); 
 hold off
 line([0 20],[0 0],'Color','black', 'LineWidth', 0.5) 
 axis([0 20 -0.95 0.2])
 xlabel('Quarter','FontSize',12)
 title('Inflation','FontSize',12)
 
 subplot(2,3,6)
 plot(dat,4*base_irfs_mpi(:,3),'b-', 'LineWidth', 1.5); 
 hold on
 plot(dat,4*SW_irfs_mpi(:,3),'r-.', 'LineWidth', 1.5); 
 plot(dat,4*IS_irfs_mpi(:,3),'k--', 'LineWidth', 1.5); 
 hold off
 line([0 20],[0 0],'Color','black', 'LineWidth', 0.5) 
 xlabel('Quarter','FontSize',12)
 title('Nominal Interest Rate','FontSize',12)

 set(fig4, 'PaperUnits', 'inches');
 scale = 2.5;
 x_width=scale*4.5; y_width=scale*3;
 set(fig4, 'PaperPosition', [0 0 x_width y_width]); 
 print('output\figure_4','-dpng','-r600') 
 